/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * All rights reserved.
 * Modified by Tim Edwards, April, 1997 for CIO-DIO48 board
 * Modified again December 20, 2000 for PCI-DIO96
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 *  diotest.c
 *
 *  This program is used to test the CIO-DIO96
 *  Linux loadable module (dio96).
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "../dio96.h"

/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

#define LONG unsigned long
#define BYTE unsigned char

char *DevAName = "/dev/dio/AP";
char *DevBName = "/dev/dio/BP";

#if CHANNELS > 2
char *DevCName = "/dev/dio/CP";
char *DevDName = "/dev/dio/DP";
#endif

/***************************************************************************
 *
 *  Main
 *
 ***************************************************************************/

int main(int argc, char **argv)
{
  int	fd, AP, BP;
#if CHANNELS > 2
  int	CP, DP;
#endif
  long  dataval;
  int   rval;
  char  locin[100];

  /* Open the device file for the counter */

  if ((AP = open(DevAName, O_RDWR)) < 0) {
    printf("error opening DIO96 device AP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-A to all ports output.\n");
  ioctl(AP, SET_MODE, OUTPUT);

  if ((BP = open(DevBName, O_RDWR)) < 0) {
    printf("error opening DIO96 device BP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-B to all ports output.\n");
  ioctl(BP, SET_MODE, OUTPUT);

#if CHANNELS > 2
  if ((CP = open(DevCName, O_RDWR)) < 0) {
    printf("error opening DIO96 device CP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-C to all ports output.\n");
  ioctl(CP, SET_MODE, OUTPUT);

  if ((DP = open(DevDName, O_RDWR)) < 0) {
    printf("error opening DIO96 device DP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-D to all ports output.\n");
  ioctl(DP, SET_MODE, OUTPUT);
#endif

  printf("sleep before read\n");
  sleep(1);
  
    dataval = (LONG)0xffffff;
    if (write(AP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if ((rval = read(AP, &dataval, 3)) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      printf("   read() returned %d, expected 3\n", rval); 
      printf("   buffer contains %0lx, expected 0xffffff\n", dataval);
      exit(1);
    }
    else printf("Data expected: 0xffffff, Data read: %0lx\n", dataval);
    sleep(1);
    dataval = (LONG)0x00;
    if (write(AP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if (read(AP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      exit(1);
    }
    else printf("Data expected: 0x000000, Data read: %0lx\n", dataval);
    sleep(1);
    ioctl(AP, SET_A, (BYTE)0xaa);
    ioctl(AP, GET_A, &dataval);
    printf("Read on A expected: 0xaa, Data read: %0lx\n", dataval);
    ioctl(AP, SET_B, (BYTE)0x55);
    ioctl(AP, GET_B, &dataval);
    printf("Read on B expected: 0x55, Data read: %0lx\n", dataval);
    ioctl(AP, SET_C, (BYTE)0xc3);
    ioctl(AP, GET_C, &dataval);
    printf("Read on C expected: 0xc3, Data read: %0lx\n", dataval);
    sleep(1);

    /* Again for device BP */

    dataval = (LONG)0xffffff;
    if (write(BP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if ((rval = read(BP, &dataval, 3)) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      printf("   read() returned %d, expected 3\n", rval); 
      printf("   buffer contains %0lx, expected 0xffffff\n", dataval);
      exit(1);
    }
    else printf("Data expected: 0xffffff, Data read: %0lx\n", dataval);
    sleep(1);
    dataval = (LONG)0x00;
    if (write(BP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if (read(BP, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      exit(1);
    }
    else printf("Data expected: 0x000000, Data read: %0lx\n", dataval);
    sleep(1);
    ioctl(BP, SET_A, (BYTE)0xaa);
    ioctl(BP, GET_A, &dataval);
    printf("Read on A expected: 0xaa, Data read: %0lx\n", dataval);
    ioctl(BP, SET_B, (BYTE)0x55);
    ioctl(BP, GET_B, &dataval);
    printf("Read on B expected: 0x55, Data read: %0lx\n", dataval);
    ioctl(BP, SET_C, (BYTE)0xc3);
    ioctl(BP, GET_C, &dataval);
    printf("Read on C expected: 0xc3, Data read: %0lx\n", dataval);
    sleep(1);

  printf("Bitwise test: \n");
  printf("Pin numbers designated as {device}P{channel}{pin} as per\n");
  printf("the National Instruments DIO-96 manual pages 2-10 and 2-11.\n");
  strcpy(locin, "APC7");
  for(;;){
     char linein[100], *newline;
     char devi, chi;
     int pinn;

     printf("Enter pin number to toggle or Q to quit [%s]: ", locin);
     fgets(linein, 100, stdin);
     if ((newline = strrchr(linein, '\n')) != NULL) *newline = '\0';
     if (strlen(linein) == 0) strcpy(linein, locin);

     if (sscanf(linein, "%c%*c%c%d", &devi, &chi, &pinn) != 3) {
	if (devi == 'Q' || devi == 'q') break;
	printf("Error in format: example--- APA0, APB5, BPC6. . ."); 
     }
     else if (pinn < 0 || pinn > 7) {
	printf("Pin number out of range 0 to 7\n");
     }
     else {
	devi = toupper(devi);
	chi = toupper(chi);
	switch (devi) {
	   case 'A': fd = AP; break;
	   case 'B': fd = BP; break;
#if CHANNELS > 2
	   case 'C': fd = CP; break;
	   case 'D': fd = DP; break;
#endif
	}
	chi -= 'A';
	if (chi >= 0 && chi <= 3 && devi >= 'A' && devi <= 'D') {
	   ioctl(fd, chi << 3 | 0x60 | pinn, &dataval);
	   dataval = 1 - (dataval & 0x01);
	   if (dataval) { 
	      ioctl(fd, chi << 3 | 0xe0 | pinn);
	      printf("H");
	   }
	   else {
	      ioctl(fd, chi << 3 | 0x80 | pinn);
	      printf("L");
	   }
	   strcpy(locin, linein);
	}
	else {
	   if (devi < 'A' || devi > 'D') printf("Device out of range A-D");
	   if (chi < 0 || chi > 3) printf("Channel out of range A-C");
	}
     }
     printf("\n");
  }

  printf("Setting all outputs to zero\n");
  dataval = (LONG)0x00;
  write(AP, &dataval, 3);
  write(BP, &dataval, 3);
#if CHANNELS > 2
  write(CP, &dataval, 3);
  write(DP, &dataval, 3);
#endif

  printf("Exiting.\n");
  close(AP);
  close(BP);
#if CHANNELS > 2
  close(CP);
  close(DP);
#endif
  return 0;
}
